function [mu_hat,mean_fit,A_hat,C_hat]= fda_code(Data,nloop,nwarmup,nknots,dim)
warning('off','MATLAB:nearlySingularMatrix')
basis=2; %basis=1 is truncated linear splines; basis=2 is cubic radial
r=1; %number of covariates for fixed effects
s=1; %number of covariates for random effects
%Data includes the following columns: 
%1 id 
%2 time
%3-(2+r) covariate x (associated) with the fixed effects
%3+r - (2+r+s) covariate z (associated) with the random effects
%3+r+s-(2+r+s+dim) response variables
% r and s are the lengths of x and z, respectively
id=Data(:,1);
nobs=length(id);
nsubjects=length(unique(id));
times_vec=Data(:,2);
%Prior variances for O-U parameters
var_a=5;
var_c=5;
%Hyperparameters for inverse gamma priors.
a1=0.001;
b1=0.001;
a2=0.001;
b2=0.001;
a3=0.001;
b3=0.001;
a4=0.001;
b4=0.001;
stt=0;
for i=1:nsubjects
   ti=times_vec(find(id==i));
   if(length(unique(ti))<length(ti))
      stt=stt+1;
   end
end
if(stt>0)
  error('Times must be distinct')
end
times=cell(1,nsubjects);
y=cell(1,nsubjects); %response variables
x=cell(1,nsubjects); %covariates associated with fixed effects
z=cell(1,nsubjects); %covariates associated with random effects
for i=1:nsubjects
    ii=find(id==i);
    times{i}=times_vec(ii);
    y{i}=Data(ii,3+r+s:2+r+s+dim);
    x{i}=Data(ii,3:2+r);
    z{i}=Data(ii,3+r:2+r+s);
end
unique_x=unique(Data(:,3:2+r),'rows');
nsub_obs=zeros(1,nsubjects);
for i=1:nsubjects
    nsub_obs(i)=sum(id==i);
end
deltat=cell(1,nsubjects); %deltat contains the time spacings
for i=1:nsubjects
    deltat{i}(1)=times{i}(1);
    for j=2:nsub_obs(i)
        deltat{i}(j)=times{i}(j)-times{i}(j-1);
    end
end
unique_times=unique(sort(times_vec));
knots=quantile(unique_times,linspace(0,1,nknots+2));
knots=knots(2:nknots+1);
if(basis==2)
   lknots=length(knots);
   radial=zeros(lknots,lknots);
   for i=1:lknots
        radial(i,:)=abs(knots(i)-knots).^3;
   end
   [svd_u,svd_s,svd_v]=svd(radial);
   sqrt_rad=svd_u*sqrt(svd_s)*svd_v';
end
%Basis functions for linear and nonlinear parts
nm=length(unique_times);
Xm=[ones(nm,1) unique_times];
Zz=zeros(nm,nknots);
if(basis==1)
   for i=1:nm
       Zz(i,:)=unique_times(i)-knots;
   end
   Zm=Zz.*(Zz>0);
else
   for i=1:nm
       Zz(i,:)=abs((unique_times(i)-knots).^3);
   end
   Zm=Zz/sqrt_rad;
end
%mu=zeros(nm,r,dim,nloop-nwarmup); 
mu_gr1=zeros(nm,dim,nloop-nwarmup); 
mu_gr2=zeros(nm,dim,nloop-nwarmup); 
%fit_gr1=zeros(nobs,dim,nloop-nwarmup);
%fit_gr2=zeros(nobs,dim,nloop-nwarmup);
fit=zeros(nobs,dim,nloop-nwarmup);
mu_temp=zeros(nm,r);
X=cell(1,nsubjects);
Z=cell(1,nsubjects);
for i=1:nsubjects
    X{i}=[ones(nsub_obs(i),1) times{i}];
    zmat=zeros(nsub_obs(i),nknots);
    if(basis==1)
       for j=1:nsub_obs(i)
           zmat(j,:)=times{i}(j)-knots;
       end
       Z{i}=zmat.*(zmat>0);
    else
       for j=1:nsub_obs(i)
           zmat(j,:)=abs((times{i}(j)-knots).^3);
       end
       Z{i}=zmat/sqrt_rad;
    end
end
beta=zeros(2,r,dim,nloop+1); %fixed effects for the mean functions
v=zeros(nknots,r,dim,nloop+1); %random effects for the mean functions
%linear part parameters for individual curves
w=zeros(2,s,dim,nsubjects,nloop+1); 
%nonlinear part params for indiv curves
u=zeros(nknots,s,dim,nsubjects,nloop+1);
var_v=ones(r,dim,nloop+1); %variances of the random effects of the mean funcs
var_w0=ones(s,dim,nloop+1); %variances of the random intercepts of the linear
                          %part of subject-specific curves
var_w1=ones(s,dim,nloop+1); %variances of the random slopes of the linear part
                         %of the subject-specific curves
var_u=ones(s,dim,nloop+1);%variances of the random effects of the nonlinear part
                        %of the subject-specific curves
%O-U parameters
na=dim^2; %number of entries in A
ng=dim*(dim-1); %number of entries in Gamma not equal to 1
nl=dim*(dim-1)/2;
nc=dim*(dim+1)/2;
%Initialize
A=zeros(dim,dim,nloop+1);
C=zeros(dim,dim,nloop+1);
Gamma=zeros(dim,dim,nloop+1);
Lambda=zeros(dim,dim,nloop+1);
L=zeros(dim,dim,nloop+1);
D=zeros(dim,dim,nloop+1);
%L(:,:,1)=[1 0;1 1];
%D(:,:,1)=[1 0;0 3];
%Acceptance probs for M-H steps
epsilon_A=zeros(1,nloop);
epsilon_C=zeros(1,nloop);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initial values for smoothing the part
%Fit a linear mixed model Y = X0*beta0 + Z0*u0 + epsilon, for each variable
%Set up design matrices X0 and Z0
%Note that lower case x and z are covariates; upper case X and Z are basis 
%functions
X0=zeros(nobs,2*r);
Z0=zeros(nobs,nknots*r+nsubjects*s*(2+nknots));
for i=1:nsubjects
    D_phi_r=zeros(r,2*r);
    D_psi_r=zeros(r,nknots*r);
    D_phi_s=zeros(s,2*s);
    D_psi_s=zeros(s,nknots*s);
    X0i=zeros(nsub_obs(i),2*r);
    Zvi=zeros(nsub_obs(i),nknots*r);
    Zwi=zeros(nsub_obs(i),2*s);
    Zui=zeros(nsub_obs(i),nknots*s);
    for j=1:nsub_obs(i)
        for l=1:r
            D_phi_r(l,1+2*(l-1):2*l)= X{i}(j,:);
            D_psi_r(l,1+nknots*(l-1):nknots*l)= Z{i}(j,:);
        end
        for m=1:s
            D_phi_s(m,1+2*(m-1):2*m)= X{i}(j,:);
            D_psi_s(m,1+nknots*(m-1):nknots*m)= Z{i}(j,:);
        end
        X0i(j,:)=x{i}(j,:)*D_phi_r;
        Zvi(j,:)=x{i}(j,:)*D_psi_r;
        Zwi(j,:)=z{i}(j,:)*D_phi_s;
        Zui(j,:)=z{i}(j,:)*D_psi_s;
    end
    ii=find(id==i);
    X0(ii,:)=X0i;
    Z0(ii,1:nknots*r)=Zvi;
    Z0(ii,nknots*r+1+(i-1)*2*s:nknots*r+i*2*s)=Zwi;
    Z0(ii,nknots*r+nsubjects*s*2+1+(i-1)*nknots*s:...
       nknots*r+nsubjects*s*2+i*nknots*s)=Zui;
end
%Assume for simplicity common variance components to all covariates:
%sigma_v^2, sigma_w^2 and sigma_u^2
vcomp_dims=zeros(1,3);
vcomp_dims(1)=nknots*r;
vcomp_dims(2)=nsubjects*s*2;
vcomp_dims(3)=nsubjects*nknots*s;
vcomp_init=chi2rnd(3,1,4); 
beta0=zeros(2*r,dim);
u0=zeros(nknots*r+nsubjects*s*(2+nknots),dim);
vcomp0=zeros(4,dim);
for k=1:dim
    %[vcomp0(:,k),beta0(:,k),u0(:,k),Is2,mat1,mat2,vec1,loglik,loops]=...
    %mixed(Data(:,2+r+s+k),X0,Z0,vcomp_dims,vcomp_init,2);
    [beta0,u0]=mixed_mcmc(Data(:,2+r+s+k),X0,Z0);
%end
%for k=1:dim
      for l=1:r
              %beta(:,l,k,1)=beta0(1+(l-1)*2:l*2,k);
              beta(:,l,k,1)=beta0(1+(l-1)*2:l*2);
              %v(:,l,k,1)=u0(1+(l-1)*nknots:l*nknots,k);
              v(:,l,k,1)=u0(1+(l-1)*nknots:l*nknots);
      end 
      %wmat=reshape(u0(nknots*r+1:nknots*r+nsubjects*2*s,k),2*s,nsubjects);
      wmat=reshape(u0(nknots*r+1:nknots*r+nsubjects*2*s),2*s,nsubjects);
      %umat=reshape(u0(nknots*r+nsubjects*2*s+1:nknots*r+nsubjects*s*...
      %            (2+nknots),k),nknots*s,nsubjects);
      umat=reshape(u0(nknots*r+nsubjects*2*s+1:nknots*r+nsubjects*s*...
                  (2+nknots)),nknots*s,nsubjects);
      for i=1:nsubjects
          for m=1:s
              w(:,m,k,i,1)=wmat(1+2*(m-1):2*m,i);
              u(:,m,k,i,1)=umat(1+nknots*(m-1):nknots*m,i);
          end
      end
      %for l=1:r
      %    var_v(l,k,1)=vcomp0(1,k);
      %end
      %for m=1:s
      %    var_w0(m,k,1)=vcomp0(2,k);
      %    var_w1(m,k,1)=vcomp0(2,k);
      %    var_u(m,k,1)=vcomp0(3,k);
      %end
end
%Initial values for O-U
%%%%%%%%%%%%%%%%%%%%%%%
res=zeros(nobs,dim,nloop+1);
for k=1:dim
    counter=0;
    for i=1:nsubjects
        for j=1:nsub_obs(i)
            counter=counter+1;
            term1=0;
            term2=0;
            for l=1:r
                term1=term1+x{i}(j,l)*(X{i}(j,:)*beta(:,l,k,1)+Z{i}(j,:)*...
                      v(:,l,k,1));
            end
            for m=1:s
                term2=term2+z{i}(j,m)*(X{i}(j,:)*w(:,m,k,i,1)+Z{i}(j,:)*...
                      u(:,m,k,i,1));
            end
            res(counter,k,1)=y{i}(j,k)-term1-term2;
        end
    end
end
options = optimset('LargeScale','off','MaxIter',10000,'MaxFunEvals',10000,'display','off');
init=normrnd(0,1,na+nl+dim,1);
%maximize log p(A,C|data)
[sol,fval,exitflag,output,grad,hessian]=fminunc(@neg_log_post_AC_ev,init,...
                             options,res(:,:,1),times_vec,id,var_a,var_c);
counter=0;
for j=1:dim
    for i=1:dim
        if (i==j)
           Gamma(i,j,1)=1;
        else
           counter=counter+1;
           Gamma(i,j,1)=sol(counter);
        end
    end
end
Lambda(:,:,1)=diag(exp(sol(ng+1:na)));
A(:,:,1)=Gamma(:,:,1)*Lambda(:,:,1)*inv(Gamma(:,:,1));
L(:,:,1)=eye(dim);
solL=sol(na+1:na+nl);
counter=0;
for j=1:dim-1
   for i=j+1:dim
       counter=counter+1;
       L(i,j,1)=solL(counter);
   end
end
solD=sol(na+nl+1:na+nl+dim);
D(:,:,1)=diag(exp(solD));
C(:,:,1)=L(:,:,1)*D(:,:,1)*L(:,:,1)';
exit=zeros(10,2);
for k=1:4
    %maximaize p(A|C,data)
    init=normrnd(0,1,na,1);
    [sol,fval,exitflag1,output,grad,hessian]=fminunc(@neg_log_post_A_ev,...
      init,options,C(:,:,1),res(:,:,1),times_vec,id,var_a);
    vcov_A=inv(hessian);
    if(k==4 & exitflag1~=1)
       vcov_A=make_pos_def(vcov_A);
    end
    counter=0;
    for j=1:dim
        for i=1:dim
            if (i==j)
               Gamma(i,j,1)=1;
            else
               counter=counter+1;
               Gamma(i,j,1)=sol(counter);
            end
        end
    end
    Lambda(:,:,1)=diag(exp(sol(ng+1:na)));
    A(:,:,1)=Gamma(:,:,1)*Lambda(:,:,1)*inv(Gamma(:,:,1));
%Maximize p(C|A, data)
    init=normrnd(0,1,nl+dim,1);
    [sol,fval,exitflag2,output,grad,hessian] =fminunc(@neg_log_post_C_fda,...
             init,options,A(:,:,1),res(:,:,1),times_vec,id,var_c);
    vcov_C=inv(hessian);
    if(k==2 & exitflag2~=1)
       vcov_C=make_pos_def(vcov_C);
    end
    L(:,:,1)=eye(dim);
    solL=sol(1:nl);
    counter=0;
    for j=1:dim-1
        for i=j+1:dim
            counter=counter+1;
            L(i,j,1)=solL(counter);
        end
    end
    solD=sol(nl+1:nl+dim);
    D(:,:,1)=diag(exp(solD));
    C(:,:,1)=L(:,:,1)*D(:,:,1)*L(:,:,1)';
    exit(k,:)=[exitflag1 exitflag2];
end
vcov_A=0.5*(vcov_A+vcov_A');
vcov_C=0.5*(vcov_C+vcov_C');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for p=1:nloop
    p
    if (dim==2)
        sigma=(det(A(:,:,p))*C(:,:,p)+(A(:,:,p)-trace(A(:,:,p))*eye(2))*...
            C(:,:,p)*(A(:,:,p)-trace(A(:,:,p))*eye(2))')...
            ./(2*trace(A(:,:,p))*det(A(:,:,p)));
    else
        sigma=lyap(A(:,:,p),-C(:,:,p));
    end
    %R=zeros(dim,1);
    %Q=zeros(dim,1);
    G=zeros(dim*(2+nknots)*r,dim*(2+nknots)*r);
    Gw=zeros(dim*(2+nknots)*s,dim*(2+nknots)*s);
    Gu=zeros(dim*(2+nknots)*s,dim*(2+nknots)*s);
    for k=1:dim
        for l=1:r
            blkG=blkdiag(zeros(2),(1/var_v(l,k,p))*eye(nknots));
            G((k-1)*r*(2+nknots)+1+(l-1)*(2+nknots):(k-1)*r*(2+nknots)+l*...
            (2+nknots),(k-1)*r*(2+nknots)+1+(l-1)*(2+nknots):...
            (k-1)*r*(2+nknots)+l*(2+nknots))=blkG;
        end
        for m=1:s
            blkGw=blkdiag(diag([1/var_w0(m,k,p) 1/var_w1(m,k,p)]),...
                  zeros(nknots));
            Gw((k-1)*s*(2+nknots)+1+(m-1)*(2+nknots):...
            (k-1)*s*(2+nknots)+m*(2+nknots),(k-1)*s*(2+nknots)+...
            1+(m-1)*(2+nknots):(k-1)*s*(2+nknots)+m*(2+nknots))=blkGw;
            blkGu=blkdiag(zeros(2),(1/var_u(m,k,p))*eye(nknots));
            Gu((k-1)*s*(2+nknots)+1+(m-1)*(2+nknots):(k-1)*s*(2+nknots)+...
               m*(2+nknots),(k-1)*s*(2+nknots)+1+(m-1)*(2+nknots):...
               (k-1)*s*(2+nknots)+m*(2+nknots))=blkGu;
        end
    end
    inv_vcov_bv=zeros(r*dim*(2+nknots),r*dim*(2+nknots));
    mean_bv=zeros(1,r*dim*(2+nknots));
    chix1_ij=zeros(dim,dim*(2+nknots)*r);
    chiz1_ij=zeros(dim,dim*(2+nknots)*s);
    for i=1:nsubjects
        wu=zeros(dim*(2+nknots)*s,1);%combined vector of w_i11 u_i11,...,
                               %w_i1s u_i1s,..., w_ip1,u_ip1,...,w_ips,u_ips
        for k=1:dim
            wu(1+(k-1)*(2+nknots)*s:k*(2+nknots)*s)=...
            [reshape(w(:,:,k,i,p),2*s,1)' reshape(u(:,:,k,i,p),nknots*s,1)']';
        end
        for j=1:nsub_obs(i)
            omega=sigma-expm(-A(:,:,p)*deltat{i}(j))*sigma*expm(-A(:,:,p)'*...
                  deltat{i}(j));
            D_phi_psi_r1=zeros(r,(2+nknots)*r);
            D_phi_psi_r2=zeros(r,(2+nknots)*r);
            D_phi_psi_s1=zeros(s,(2+nknots)*s);
            D_phi_psi_s2=zeros(s,(2+nknots)*s);
            for l=1:r
                D_phi_psi_r1(l,1+(l-1)*(2+nknots):l*(2+nknots))=...
                [X{i}(j,:) Z{i}(j,:)];
            end
            for m=1:s
                D_phi_psi_s1(m,1+(m-1)*(2+nknots):m*(2+nknots))=...
                [X{i}(j,:) Z{i}(j,:)];
            end
            for k=1:dim
                chix1_ij(k,1+(k-1)*(2+nknots)*r:k*(2+nknots)*r)=...
                       x{i}(j,:)*D_phi_psi_r1;
                chiz1_ij(k,1+(k-1)*(2+nknots)*s:k*(2+nknots)*s)=...
                       z{i}(j,:)*D_phi_psi_s1;
            end
            if(j==1)
               R=y{i}(j,:)'-chiz1_ij*wu;
               diff_chi=chix1_ij;
            else
               for m=1:s
                   D_phi_psi_s2(m,1+(m-1)*(2+nknots):m*(2+nknots))=...
                   [X{i}(j-1,:) Z{i}(j-1,:)];
               end
               for l=1:r
                   D_phi_psi_r2(l,1+(l-1)*(2+nknots):l*(2+nknots))=...
                   [X{i}(j-1,:) Z{i}(j-1,:)];
               end
               chix2_ij=zeros(dim,dim*(2+nknots)*r);
               chiz2_ij=zeros(dim,dim*(2+nknots)*s);
               for k=1:dim
                   chix2_ij(k,1+(k-1)*(2+nknots)*r:k*(2+nknots)*r)=...
                       x{i}(j-1,:)*D_phi_psi_r2;
                   chiz2_ij(k,1+(k-1)*(2+nknots)*s:k*(2+nknots)*s)=...
                       z{i}(j-1,:)*D_phi_psi_s2;
               end
               R=y{i}(j,:)'-chiz1_ij*wu-expm(-A(:,:,p)*deltat{i}(j))*...
                 (y{i}(j-1,:)'-chiz2_ij*wu);
               diff_chi=chix1_ij-expm(-A(:,:,p)*deltat{i}(j))*chix2_ij;
            end
            inv_vcov_bv=inv_vcov_bv+diff_chi'*inv(omega)*diff_chi;
            mean_bv=mean_bv+R'*inv(omega)*diff_chi;
        end
    end
    inv_vcov_bv=inv_vcov_bv+G;
    vcov_bv=inv(inv_vcov_bv);
    vcov_bv=.5*(vcov_bv+vcov_bv');
    mean_bv=mean_bv*vcov_bv;
    bv=mvnrnd(mean_bv,vcov_bv);
    bv_mat=reshape(bv',2+nknots,r*dim);
    counter=0;
    for k=1:dim
        for l=1:r
            counter=counter+1;
            beta(:,l,k,p+1)=bv_mat(1:2,counter);
            v(:,l,k,p+1)=bv_mat(3:2+nknots,counter);
        end
    end
    for i=1:nsubjects
        inv_vcov_wu=zeros(dim*(2+nknots)*s,dim*(2+nknots)*s);
        mu_wu=zeros(1,dim*(2+nknots)*s);
        for j=1:nsub_obs(i)
            omega=sigma-expm(-A(:,:,p)*deltat{i}(j))*sigma*...
                  expm(-A(:,:,p)'*deltat{i}(j));
            D_phi_psi_r1=zeros(r,(2+nknots)*r);
            D_phi_psi_r2=zeros(r,(2+nknots)*r);
            D_phi_psi_s1=zeros(s,(2+nknots)*s);
            D_phi_psi_s2=zeros(s,(2+nknots)*s);
            for l=1:r
                D_phi_psi_r1(l,1+(l-1)*(2+nknots):l*(2+nknots))=...
                [X{i}(j,:) Z{i}(j,:)];
            end
            for m=1:s
                D_phi_psi_s1(m,1+(m-1)*(2+nknots):m*(2+nknots))=...
                [X{i}(j,:) Z{i}(j,:)];
            end
            for k=1:dim
                chix1_ij(k,1+(k-1)*(2+nknots)*r:k*(2+nknots)*r)=...
                       x{i}(j,:)*D_phi_psi_r1;
                chiz1_ij(k,1+(k-1)*(2+nknots)*s:k*(2+nknots)*s)=...
                       z{i}(j,:)*D_phi_psi_s1;
            end
            if(j==1)
              Q=y{i}(j,:)'-chix1_ij*bv';
              diff_chi=chiz1_ij;
            else
               for m=1:s
                   D_phi_psi_s2(m,1+(m-1)*(2+nknots):m*(2+nknots))=...
                   [X{i}(j-1,:) Z{i}(j-1,:)];
               end
               for l=1:r
                   D_phi_psi_r2(l,1+(l-1)*(2+nknots):l*(2+nknots))=...
                   [X{i}(j-1,:) Z{i}(j-1,:)];
               end
               chix2_ij=zeros(dim,dim*(2+nknots)*r);
               chiz2_ij=zeros(dim,dim*(2+nknots)*s);
               for k=1:dim
                   chix2_ij(k,1+(k-1)*(2+nknots)*r:k*(2+nknots)*r)=...
                       x{i}(j-1,:)*D_phi_psi_r2;
                   chiz2_ij(k,1+(k-1)*(2+nknots)*s:k*(2+nknots)*s)=...
                       z{i}(j-1,:)*D_phi_psi_s2;
               end
               Q=y{i}(j,:)'-chix1_ij*bv'-expm(-A(:,:,p)*deltat{i}(j))*(...
                 y{i}(j-1,:)'-chix2_ij*bv');
               diff_chi=chiz1_ij-expm(-A(:,:,p)*deltat{i}(j))*chiz2_ij;
            end
            inv_vcov_wu=inv_vcov_wu+diff_chi'*inv(omega)*diff_chi;
            mu_wu=mu_wu+Q'*inv(omega)*diff_chi;
        end
        inv_vcov_wu=inv_vcov_wu+Gw+Gu;
        vcov_wu=inv(inv_vcov_wu);
        vcov_wu=.5*(vcov_wu+vcov_wu');
        mu_wu=mu_wu*vcov_wu;
        wu=mvnrnd(mu_wu,vcov_wu);
        wu_mat=reshape(wu',2+nknots,s*dim);
        counter=0;
        for k=1:dim
            for m=1:s
                counter=counter+1;
                w(:,m,k,i,p+1)=wu_mat(1:2,counter);
                u(:,m,k,i,p+1)=wu_mat(3:2+nknots,counter);
            end
        end
    end
    for k=1:dim
        for l=1:r
            var_v(l,k,p+1)=1/gamrnd(nknots/2+a1,1/(b1+0.5*v(:,l,k,p+1)'*...
                                    v(:,l,k,p+1)));
        end
        for m=1:s
            ss1=0;
            ss2=0;
            ss3=0;
            for i=1:nsubjects
                ss1=ss1+w(1,m,k,i,p+1)^2;
                ss2=ss2+w(2,m,k,i,p+1)^2;
                ss3=ss3+u(:,m,k,i,p+1)'*u(:,m,k,i,p+1);
            end
            var_w0(m,k,p+1)=1/gamrnd(nsubjects/2+a2,1/(0.5*ss1+b2));
            var_w1(m,k,p+1)=1/gamrnd(nsubjects/2+a3,1/(0.5*ss2+b3));
            var_u(m,k,p+1)=1/gamrnd(nsubjects*nknots/2+a4,1/(0.5*ss3+b4));
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %O-U process
    %Compute residuals
    for k=1:dim
        counter=0;
        for i=1:nsubjects
            for j=1:nsub_obs(i)
                counter=counter+1;
                term1=0;
                term2=0;
                for l=1:r
                    term1=term1+x{i}(j,l)*(X{i}(j,:)*beta(:,l,k,p+1)+...
                          Z{i}(j,:)*v(:,l,k,p+1));
                end
                for m=1:s
                    term2=term2+z{i}(j,m)*(X{i}(j,:)*w(:,m,k,i,p+1)+...
                          Z{i}(j,:)*u(:,m,k,i,p+1));
                end
                res(counter,k,p+1)=y{i}(j,k)-term1-term2;
            end
        end
    end
    init=normrnd(0,1,na,1);
    %maximize log p(A|C,data)
    %[sol_GL,fval,exitflag,output,grad_GL,hessian_GL]=...
    % fminunc(@neg_log_post_A_ev,init,options,C(:,:,p),res(:,:,p),...
    %         times_vec,id,var_a);
    % [G1,q]=chol(hessian_GL);
    % if(q==0)
    %    vcov_GL=1.2*inv(hessian_GL);
    % else
    %    vcov_GL=1.2*eye(na);
    % end
    %propose new parameter values for Gamma and log(Lambda)
     par_Gam_old=zeros(ng,1);
     counter=0;
     for j=1:dim
         for i=1:dim
             if (i~=j)
                counter=counter+1;
                par_Gam_old(counter)=Gamma(i,j,p);
             end
         end
     end
     par_log_Lam_old=log(diag(Lambda(:,:,p)));
     par_GL_old=[par_Gam_old' par_log_Lam_old']';
     par_GL_new=mvnrnd(par_GL_old,(5.76/na)*vcov_A)';
     par_Gam_new=par_GL_new(1:ng);
     counter=0;
     for j=1:dim
         for i=1:dim
             if (i==j)
                Gamma(i,j,p+1)=1;
             else
                counter=counter+1;
                Gamma(i,j,p+1)=par_Gam_new(counter);
             end
         end
     end
     par_log_Lam_new=par_GL_new(ng+1:na);
     par_Lam_new=exp(par_log_Lam_new);
     Lambda(:,:,p+1)=diag(par_Lam_new);
     A(:,:,p+1)=Gamma(:,:,p+1)*Lambda(:,:,p+1)*inv(Gamma(:,:,p+1));
%Evaluate log p(A|C,data) at old and new values of A
     log_post_A_old=log_post_A_ev(res(:,:,p),times_vec,id,Gamma(:,:,p),...
                    Lambda(:,:,p),C(:,:,p),var_a);
     log_post_A_new=log_post_A_ev(res(:,:,p),times_vec,id,Gamma(:,:,p+1),...
                    Lambda(:,:,p+1),C(:,:,p),var_a);
     %Evaluating the proposal density at the old and new values
     %vcov_GL_inv=inv(vcov_GL);
     %log_prop_GL_new=-0.5*(par_GL_new-sol_GL)'*vcov_GL_inv*(par_GL_new-sol_GL);
     %log_prop_GL_old=-0.5*(par_GL_old-sol_GL)'*vcov_GL_inv*(par_GL_old-sol_GL);
     met_rat_A=log_post_A_new-log_post_A_old;
     epsilon_A(p)=min(1,exp(met_rat_A));
     %epsilon_A(p)
     uu=rand;
     if(uu > epsilon_A(p))
        A(:,:,p+1)=A(:,:,p);
        Gamma(:,:,p+1)=Gamma(:,:,p);
        Lambda(:,:,p+1)=Lambda(:,:,p);
     end
    init=normrnd(0,1,nl+dim,1);
    %maximize log p(C|A,data)
    %[sol_LD,fval,exitflag,output,grad_LD,hessian_LD] = ...
    % fminunc(@neg_log_post_C_fda,init,options,A(:,:,p+1),res(:,:,p),...
    %       times_vec,id,var_c);
    %[G,q]=chol(hessian_LD);
    %if(q==0)
    %   vcov_LD=inv(hessian_LD);
    %else
    %   vcov_LD=eye(nl+dim);
    %end
    %propose new parameter values for L and log(D)
    par_L_old=zeros(nl,1);
    counter=0;
    for j=1:dim-1
        for i=j+1:dim
            counter=counter+1;
            par_L_old(counter)=L(i,j,p);
        end
    end
    par_D_old=log(diag(D(:,:,p)));
    par_LD_old=[par_L_old' par_D_old']';
    %par_LD_new=mvnrnd(sol_LD,vcov_LD)';
    par_LD_new=mvnrnd(par_LD_old,(5.76/nc)*vcov_C)';
    %par_LD_new=mvnrnd(par_LD_old,(5.76/nc)*eye(nc))';
    par_L_new=par_LD_new(1:nl);
    L(:,:,p+1)=eye(dim);
    counter=0;
    for j=1:dim-1
        for i=j+1:dim
            counter=counter+1;
            L(i,j,p+1)=par_L_new(counter);
            %par_L_old(counter)=L(i,j,p);
        end
    end
    par_D_new=par_LD_new(nl+1:nl+dim);
    D(:,:,p+1)=diag(exp(par_D_new));
    C(:,:,p+1)=L(:,:,p+1)*D(:,:,p+1)*L(:,:,p+1)';
    %Evaluate log p(C|A,data) at old and new values of L and D
    log_post_C_old=log_post_C_fda(res(:,:,p),times_vec,id,A(:,:,p+1),...
                   L(:,:,p),D(:,:,p),var_c);
    log_post_C_new=log_post_C_fda(res(:,:,p),times_vec,id,A(:,:,p+1),...
                   L(:,:,p+1),D(:,:,p+1),var_c);
    %Evaluating the proposal density at the old and new values
    %vcov_LD_inv=inv(vcov_LD);
    %log_prop_LD_new=-0.5*(par_LD_new-sol_LD)'*vcov_LD_inv*(par_LD_new-sol_LD);
    %log_prop_LD_old=-0.5*(par_LD_old-sol_LD)'*vcov_LD_inv*(par_LD_old-sol_LD);
    %met_rat_C=log_post_C_new-log_post_C_old+log_prop_LD_old-log_prop_LD_new;
    met_rat_C=log_post_C_new-log_post_C_old;
    epsilon_C(p)=min(1,exp(met_rat_C));
    uu=rand;
    if(uu > epsilon_C(p))
       C(:,:,p+1)=C(:,:,p);
       L(:,:,p+1)=L(:,:,p);
       D(:,:,p+1)=D(:,:,p);
    end
   %%%%%%%%%%%
    if(p>nwarmup)
       for j=1:dim
               for l=1:r
                   mu(:,l,j,p-nwarmup)=Xm*beta(:,l,j,p)+Zm*v(:,l,j,p);
                   mu_temp(:,l)=Xm*beta(:,l,j,p)+Zm*v(:,l,j,p);
               end
               %mu_gr1(:,j,p-nwarmup)=sum(repmat(unique_x(1,:),nm,1).*mu_temp,2);
               %mu_gr2(:,j,p-nwarmup)=sum(repmat(unique_x(2,:),nm,1).*mu_temp,2);
               for i=1:nsubjects
                   mu_temp1=zeros(nsub_obs(i),r);
                   temp2=zeros(nsub_obs(i),s);
                   for l=1:r
                       mu_temp1(:,l)=X{i}*beta(:,l,j,p)+Z{i}*v(:,l,j,p);
                   end
                   for m=1:s
                       temp2(:,m)=X{i}*w(:,m,j,i,p)+Z{i}*u(:,m,j,i,p);
                   end
                   fit(find(id==i),j,p-nwarmup)=sum(x{i}.*mu_temp1,2)+...
                       sum(z{i}.*temp2,2);
                       %X{i}*w(:,1,j,i,p)+Z{i}*u(:,1,j,i,p);
               end
       end
    end

end %end of main loop
%mean_mu_gr1=mean(mu_gr1,3);
%mean_mu_gr2=mean(mu_gr2,3);
%mean_fit=mean(fit,3);
%mu_true1=[0.5*normcdf((unique_times-5)/1.3,0,1)'; 0.5+0.5*...
%         normcdf((unique_times-5)/1.3,0,1)']';
%mu_true2=[0.5*exp(-0.3*unique_times)'; 0.75+0.5*exp(-0.3*unique_times)']'; 
%a1=8 ;a2=2;
%b1=8 ;b2=1.5;
%mu_true(:,1)=b1*sin(-b2*unique_times)';
%mu_true(:,2)=a1*sin(-a2*unique_times)';
%A_true=[.5 -.3; -.3 .5];
%C_true=[10 5; 5 10];

mean_fit=mean(fit,3);
mu_hat=squeeze(mean(mu,4));
%for j=1:dim
%    subplot(1,dim,j)
%    plot(unique_times,mu_true(:,j),'LineWidth',5,'Color','b')
%    hold on
%    plot(unique_times,mu_hat(:,j),'LineWidth',3,'Color','g')
%    hold off
%end

%for j=1:dim
%    mu1(:,j)=Xm*beta(:,j,1)+Zm*v(:,j,1);
%end
%for j=1:dim
%    subplot(1,dim,j)
%    plot(unique_times,mu1(:,j),'LineWidth',2,'Color','r')
%    hold on
%    plot(unique_times,mu_true(:,j),'LineWidth',2,'Color','b')
%    hold off
%end
%for j=1:dim
%    subplot(1,dim,j)
%    plot(unique_times,mean_mu_gr1(:,j),unique_times,mean_mu_gr2(:,j),...
%        'LineWidth',2,'Color','r')
%    hold on
    %if(j==1)
    %   plot(unique_times,mu_true1(:,1),unique_times,mu_true1(:,2),...
    %       'LineWidth',2,'Color','g')
    %else
    %   plot(unique_times,mu_true2(:,1),unique_times,mu_true2(:,2),...
    %       'LineWidth',2,'Color','g')
    %end
    %for i=1:nsubjects
    %    plot(times{i},y{i}(:,j),'.','MarkerSize',8,'Color','r')
    %    hold on
    %    %plot(times{i},mean_fit(find(id==i),j),times{i},mean_fit(find(id==i),j))
    %    plot(times{i},mean_fit(find(id==i),j))
    %end
%end
A_hat=mean(A(:,:,nwarmup+1:nloop),3);
C_hat=mean(C(:,:,nwarmup+1:nloop),3);
