function neg_logp_AC=neg_log_post_AC_ev(par,x,tt,id,var_a,var_c)
%A is parameterized as G*Lambda*G^{-1} where the columns of G are the 
%eigenvectors of A, and Lambda is the diagonal matrix of eigenvalues
%C is parameterized as L*D*L', the modified Cholesky decomposition
%par is a vector containing the entries of G, Lambda, L and D (in this order)
%tt is the times for all subjects, in ascending order within each subject
%x is the process values for all subjects and variables, corresponding
%to tt.
%id is the subject id's.
nsubjects=length(unique(id));
[nobs,dim]=size(x);
%ng is number of elements of G not equal 1
%nl is the number of elements of L (unit lower triangular)
ng=dim*(dim-1);
na=dim^2; %total number of parameters used in the decomposition of A (ng+dim)
nl=dim*(dim-1)/2;
G=zeros(dim,dim);
counter=0;
for j=1:dim
    for i=1:dim
        if (i==j)
           G(i,j)=1;
        else
           counter=counter+1;
           G(i,j)=par(counter);
        end
    end
end
%Constrain eigenvalues to be positive
Lambda=diag(exp(par(ng+1:na)));
A=G*Lambda*inv(G);
parL=par(na+1:na+nl);
L=eye(dim);
counter=0;
for j=1:dim-1
    for i=j+1:dim
        counter=counter+1;
        L(i,j)=parL(counter);
    end
end
parD=exp(par(na+nl+1:na+nl+dim));
D=diag(parD);
C=L*D*L';
if (dim==2)
    sig=(det(A)*C+(A-trace(A)*eye(2))*C*(A-trace(A)*eye(2))')...
        ./(2*trace(A)*det(A));
else
    sig=lyap(A,-C);
end
nsub_obs=zeros(1,nsubjects);
for i=1:nsubjects
    nsub_obs(i)=sum(id==i);
end
sum1=0;
sum2=0;
for i=1:nsubjects
    ti=tt(find(id==i));
    xi=x(find(id==i),:);
    for j=2:nsub_obs(i);
        dt=ti(j)-ti(j-1);
        temp=expm(-A*dt);
        mu=temp*xi(j-1,:)';
        omega=sig-temp*sig*temp';
        sum1=sum1+log(det(omega));
        y=xi(j,:)'-mu;
        sum2=sum2+y'*inv(omega)*y;
    end
end
sum3=0;
for i=1:dim
    for j=1:dim
        if(i~=j)
           sum3=sum3+G(i,j)^2;
        end
    end
end
logp_AC=-0.5*(sum1+sum2)-sum(log(diag(Lambda)))-...
         0.5*sum(log(diag(Lambda)).^2)/var_a-0.5*sum3/var_a...
         -0.5*sum(sum(L.^2))/var_c-sum(log(diag(D)))-...
         0.5*sum(log(diag(D)).^2)/var_c;
neg_logp_AC=-logp_AC;
